@@ -5,6 +5,8 @@ from matplotlib.mlab import psd, csd
 from scipy.optimize import minimize
 from scipy import signal
 
+from numpy.fft import fft, fftfreq
+
 class tfest:
     def __init__(self, u, y):
         self.u = u
@@ -28,35 +30,51 @@ class tfest:
         zeros = x[:nzeros]
         poles = x[nzeros:]
         risp = np.array([
-            (1+sum([a*1j*s**i for i, a in enumerate(zeros)]))/(1+sum([b*1j*s**i for i, b in enumerate(poles)]))
+            (sum([a*(1j*s)**i for i, a in enumerate(zeros)]))/(sum([b*(1j*s)**i for i, b in enumerate(poles)]))
             for s in freq])
-        return np.linalg.norm(H-risp).sum() # + np.abs(x.sum())
+        return np.square(np.linalg.norm((risp-H).reshape(-1, 1), axis=1)).sum() #+ np.abs(x).sum()
 
-    def frequency_response(self):
+    def frequency_response(self, method="density", time=1):
         """
+        method: "fft" or "density"
+        time: time for fft
+
         return: frequency response and frequency
         """
-        cross_sd, frequency = csd(self.y, self.u)
-        power_sd, _ = psd(self.u)
+        if method == "fft":
+            u_f = fft(self.u)
+            y_f = fft(self.y)
+            u_no_zero = u_f[np.nonzero(u_f)]
+            y_no_zero = y_f[np.nonzero(u_f)]
+            frequency = fftfreq(len(u_no_zero), time/len(u_no_zero))
+            H = y_no_zero/u_no_zero
+        elif method == "density":
+            cross_sd, frequency = csd(self.y, self.u)
+            power_sd, _ = psd(self.u)
+            H = cross_sd/power_sd
         self.frequency = frequency
-        self.H = cross_sd/power_sd
+        self.H = H
         return self.H, frequency
 
-    def estimate(self, npoles, nzeros, init_value=1, options={'xatol': 1e-4, 'disp': True}):
+    def estimate(self, nzeros, npoles, init_value=1, options={'xatol': 1e-2, 'disp': True}, method="density", time=1):
         """
         npoles: number of poles
         nzeros: number of zeros
         init_value: initial value for optimization
         options: options for scipy.optimize.minimize
+        method: "fft" or "density"
+        time: time for fft
 
         return: scipy.optimize.minimize.OptimizeResult
         """
+        npoles += 1
+        nzeros += 1
         self.npoles = npoles
         self.nzeros = nzeros
         self.init_value = init_value
 
         x0 = [init_value]*(npoles+nzeros)
-        H, frequency = self.frequency_response()
+        H, frequency = self.frequency_response(method=method, time=time)
         pass_to_loss = lambda x: self.loss(x, nzeros, frequency, H)
         self.res = minimize(pass_to_loss, x0, method='nelder-mead', options=options)
         return self.res
@@ -69,8 +87,6 @@ class tfest:
             raise Exception("Please run .estimate(npoles, nzeros) before plotting.")
         zeros = list(reversed(self.res.x[:self.nzeros]))
         poles = list(reversed(self.res.x[self.nzeros:]))
-        zeros[-1] += 1
-        poles[-1] += 1
         return signal.lti(zeros, poles)
 
     def plot_bode(self):
@@ -82,9 +98,13 @@ class tfest:
         tf = self.get_transfer_function()
         w, mag, phase = tf.bode()
         plt.figure()
+        plt.title('Bode magnitude plot')
         plt.semilogx(w, mag)
+        plt.grid()
         plt.figure()
+        plt.title('Bode phase plot')
         plt.semilogx(w, phase)
+        plt.grid()
         plt.show()
            
     def plot(self):
@@ -96,9 +116,10 @@ class tfest:
         zeros = self.res.x[:self.nzeros]
         poles = self.res.x[self.nzeros:]
         risp = np.array([
-            (1+sum([a*1j*s**i for i, a in enumerate(zeros)]))/(1+sum([b*1j*s**i for i, b in enumerate(poles)]))
+            (sum([a*(1j*s)**i for i, a in enumerate(zeros)]))/(sum([b*(1j*s)**i for i, b in enumerate(poles)]))
             for s in self.frequency])
-        plt.plot(-np.log(risp), label="estimation")
+        plt.plot(np.log(risp), label="estimation")
         plt.plot(np.log(self.H), label="train data")
         plt.legend(loc="upper right")
+        plt.grid()
         plt.show()
\ No newline at end of file
